/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/*************************************************************************************************
 * A function to return content widgets and some miscellaneous content widgets.
 *************************************************************************************************/
module mde.gui.widget.miscContent;

import mde.gui.widget.Widget;
import mde.gui.exception;
import mde.gui.widget.textContent;
import mde.gui.widget.layout;

import mde.gui.renderer.IRenderer;
import mde.gui.content.Content;
import Items = mde.gui.content.Items;

/*************************************************************************************************
 * A function which uses Items.get (data.strings[0]) to get a content and creates a widget from
 * data.ints[1]. The first item in each ints and strings is removed before passing data to the new
 * widget.
 * 
 * The function only takes an IContent parameter to satisfy createWidget; it's value is ignored.
 *************************************************************************************************/
IChildWidget addContent (IWidgetManager mgr, widgetID id, WidgetData data, IContent) {
    if (data.ints.length < 2 || data.strings.length < 1) throw new WidgetDataException;
    char[] cItem = data.strings[0];
    data.strings = data.strings[1..$];
    data.ints    = data.ints   [1..$];
    return mgr.makeWidget (id, data, Items.get (cItem));
}

/*************************************************************************************************
 * A function which returns the most appropriate content editing widget.
 *
 * Widgets which can be returned: BoolContentWidget (toggle button), ValueContentWidget (generic
 * text-box editor), DisplayContentWidget (generic text label).
 *************************************************************************************************/
IChildWidget editContent (IWidgetManager mgr, widgetID id, WidgetData data, IContent c) {
    if (c is null) throw new ContentException;
    if (cast(BoolContent) c)
        return new BoolContentWidget(mgr,id,data,c);
    else if (cast(ValueContent) c)
	return new ValueContentWidget(mgr,id,data,c);
    else if (cast(ContentList) c)
	return new ContentListWidget(mgr,id,data,c);
    else        // generic uneditable option
        return new DisplayContentWidget(mgr,id,data,c);
}

/// Editable boolean widget
class BoolContentWidget : AButtonWidget
{
    this (IWidgetManager mgr, widgetID id, WidgetData data, IContent c) {
        WDCheck(data, 1);
        content = cast(BoolContent) c;
        if (!content) throw new ContentException ();
        wdimPair s = mgr.renderer.getToggleSize;
        w = mw = s.x;
        h = mh = s.y;
        super (mgr, id, data);
    }
    
    void draw () {
        mgr.renderer.drawToggle (x,y, content(), pushed);
    }
    
    void activated () {
        content = !content();
    }
    
protected:
    BoolContent content;
}

